@@ -1,4 +1,4 @@
-#     Copyright 2024, Kay Hayen, mailto:kay.hayen@gmail.com find license text at end of file
+#     Copyright 2025, Kay Hayen, mailto:kay.hayen@gmail.com find license text at end of file
 
 
 """ Collection of information for reports and their writing.
@@ -16,9 +16,11 @@ from nuitka import TreeXML
 from nuitka.__past__ import unicode
 from nuitka.build.DataComposerInterface import getDataComposerReportValues
 from nuitka.build.SconsUtils import readSconsErrorReport
+from nuitka.code_generation.ConstantCodes import getDistributionMetadataValues
 from nuitka.containers.OrderedSets import OrderedSet
 from nuitka.freezer.IncludedDataFiles import getIncludedDataFiles
 from nuitka.freezer.IncludedEntryPoints import getStandaloneEntryPoints
+from nuitka.freezer.Standalone import getRemovedUsedDllsInfo
 from nuitka.importing.Importing import getPackageSearchPath
 from nuitka.importing.Recursion import getRecursionDecisions
 from nuitka.ModuleRegistry import (
@@ -28,6 +30,7 @@ from nuitka.ModuleRegistry import (
     getModuleOptimizationTimingInfos,
 )
 from nuitka.Options import (
+    getCompilationMode,
     getCompilationReportFilename,
     getCompilationReportTemplates,
     getCompilationReportUserData,
@@ -37,10 +40,15 @@ from nuitka.Options import (
 from nuitka.OutputDirectories import (
     getResultRunFilename,
     getSourceDirectoryPath,
+    hasMainModule,
 )
 from nuitka.plugins.Plugins import getActivePlugins
 from nuitka.PythonFlavors import getPythonFlavorName
-from nuitka.PythonVersions import getSystemPrefixPath, python_version_full_str
+from nuitka.PythonVersions import (
+    getLaunchingSystemPrefixPath,
+    getSystemPrefixPath,
+    python_version_full_str,
+)
 from nuitka.Tracing import ReportingSystemExit, reports_logger
 from nuitka.utils.Distributions import (
     getDistributionInstallerName,
@@ -49,10 +57,23 @@ from nuitka.utils.Distributions import (
     getDistributionsFromModuleName,
     getDistributionVersion,
 )
-from nuitka.utils.FileOperations import getReportPath, putTextFileContents
+from nuitka.utils.FileOperations import (
+    getReportPath,
+    putBinaryFileContents,
+    putTextFileContents,
+)
 from nuitka.utils.Jinja2 import getTemplate
 from nuitka.utils.MemoryUsage import getMemoryInfos
-from nuitka.utils.Utils import getArchitecture, getOS
+from nuitka.utils.Utils import (
+    getArchitecture,
+    getLinuxDistribution,
+    getMacOSRelease,
+    getOS,
+    getWindowsRelease,
+    isLinux,
+    isMacOS,
+    isWin32OrPosixWindows,
+)
 from nuitka.Version import getCommercialVersion, getNuitkaVersion
 
 
@@ -60,7 +81,8 @@ def _getReportInputData(aborted):
     """Collect all information for reporting into a dictionary."""
 
     # used with locals for laziness and these are to populate a dictionary with
-    # many entries, pylint: disable=possibly-unused-variable,too-many-locals
+    # many entries,
+    # pylint: disable=possibly-unused-variable,too-many-branches,too-many-locals,too-many-statements
 
     module_names = tuple(module.getFullName() for module in getDoneModules())
 
@@ -171,6 +193,11 @@ def _getReportInputData(aborted):
 
         module_exclusions[_using_module_name][_module_name] = _reason
 
+    included_metadata = dict(
+        (distribution_name, meta_data_value.reasons)
+        for distribution_name, meta_data_value in getDistributionMetadataValues()
+    )
+
     memory_infos = getMemoryInfos()
 
     python_exe = sys.executable
@@ -180,6 +207,18 @@ def _getReportInputData(aborted):
     os_name = getOS()
     arch_name = getArchitecture()
 
+    # Record system encoding, spell-checker: ignore getfilesystemencoding
+    filesystem_encoding = sys.getfilesystemencoding()
+
+    if isWin32OrPosixWindows():
+        os_release = str(getWindowsRelease())
+    elif isLinux():
+        os_release = "-".join(x for x in getLinuxDistribution() if x)
+    elif isMacOS():
+        os_release = getMacOSRelease()
+    else:
+        os_release = "unknown"
+
     nuitka_version = getNuitkaVersion()
     nuitka_commercial_version = getCommercialVersion() or "not installed"
 
@@ -191,9 +230,18 @@ def _getReportInputData(aborted):
 
     data_composer = getDataComposerReportValues()
 
-    output_run_filename = os.path.abspath(getResultRunFilename(onefile=isOnefileMode()))
+    if hasMainModule():
+        output_run_filename = os.path.abspath(
+            getResultRunFilename(onefile=isOnefileMode())
+        )
+        scons_error_report_data = readSconsErrorReport(
+            source_dir=getSourceDirectoryPath()
+        )
+    else:
+        scons_error_report_data = {}
+        output_run_filename = "failed too early"
 
-    scons_error_report_data = readSconsErrorReport(source_dir=getSourceDirectoryPath())
+    compilation_mode = getCompilationMode()
 
     return dict(
         (var_name, var_value)
@@ -213,7 +261,7 @@ def _getReportPathPrefixes():
     if _report_prefixes is None:
         _report_prefixes = []
 
-        sys_prefix = os.getenv("NUITKA_SYS_PREFIX", sys.prefix)
+        sys_prefix = getLaunchingSystemPrefixPath() or sys.prefix
         real_sys_prefix = getSystemPrefixPath()
 
         if real_sys_prefix != sys_prefix:
@@ -230,7 +278,8 @@ def _getCompilationReportPath(path):
 
 
 def _addModulesToReport(root, report_input_data, diffable):
-    # Many details to work with, pylint: disable=too-many-branches,too-many-locals
+    # Many details to work with,
+    # pylint: disable=too-many-branches,too-many-locals,too-many-statements
 
     for module_name in report_input_data["module_names"]:
         active_module_info = report_input_data["module_inclusion_infos"][module_name]
@@ -271,7 +320,12 @@ def _addModulesToReport(root, report_input_data, diffable):
                     )
                 influence_xml_node.attrib["result"] = str(condition_result).lower()
             elif influence == "variable-used":
-                variable_name, condition_tags_used, variable_value = detail
+                (
+                    variable_name,
+                    condition_tags_used,
+                    variable_value,
+                    config_module_name,
+                ) = detail
 
                 influence_xml_node.attrib["variable"] = variable_name
                 if condition_tags_used:
@@ -279,6 +333,10 @@ def _addModulesToReport(root, report_input_data, diffable):
                         condition_tags_used
                     )
                 influence_xml_node.attrib["value"] = variable_value
+
+                if module_name != config_module_name:
+                    influence_xml_node.attrib["config_module"] = config_module_name
+
             elif influence == "parameter-used":
                 parameter_name, condition_tags_used, parameter_value = detail
 
@@ -309,6 +367,20 @@ def _addModulesToReport(root, report_input_data, diffable):
                 "volatile" if diffable else "%.2f" % timing_info.time_used
             )
 
+            if timing_info.micro_passes:
+                timing_xml_node.attrib["micro_passes"] = str(timing_info.micro_passes)
+
+            if timing_info.merge_counts:
+                merged_total = 0
+
+                for branch_count, merge_count in timing_info.merge_counts.items():
+                    merged_total += branch_count * merge_count
+
+                max_merge_size = max(timing_info.merge_counts)
+
+                timing_xml_node.attrib["max_branch_merge"] = str(max_merge_size)
+                timing_xml_node.attrib["merged_total"] = str(merged_total)
+
             module_xml_node.append(timing_xml_node)
 
         distributions = report_input_data["module_distribution_usages"][module_name]
@@ -349,13 +421,36 @@ def _addModulesToReport(root, report_input_data, diffable):
             "module_usages",
         )
 
-        for used_module in report_input_data["module_usages"][module_name]:
+        for count, used_module in enumerate(
+            report_input_data["module_usages"][module_name]
+        ):
+            # We don't want to see those parent imports, unless they have
+            # an effect.
+            if used_module.reason == "import path parent":
+                while True:
+                    count += 1
+                    next_used_module = report_input_data["module_usages"][module_name][
+                        count
+                    ]
+
+                    if next_used_module.reason != "import path parent":
+                        break
+
+                exclusion_reason = report_input_data["module_exclusions"][
+                    module_name
+                ].get(next_used_module.module_name)
+
+                if exclusion_reason is None or next_used_module.finding != "not-found":
+                    continue
+
             module_usage_node = TreeXML.appendTreeElement(
                 used_modules_xml_node,
                 "module_usage",
                 name=used_module.module_name.asString(),
                 finding=used_module.finding,
                 line=str(used_module.source_ref.getLineNumber()),
+                # TODO: Add reason in a hotfix.
+                # reason=used_module.reason,
             )
 
             exclusion_reason = report_input_data["module_exclusions"][module_name].get(
@@ -426,6 +521,7 @@ def writeCompilationReport(report_filename, report_input_data, diffable):
         "nuitka-compilation-report",
         nuitka_version=report_input_data["nuitka_version"],
         nuitka_commercial_version=report_input_data["nuitka_commercial_version"],
+        mode=report_input_data["compilation_mode"],
         completion=completion,
     )
 
@@ -518,6 +614,22 @@ def writeCompilationReport(report_filename, report_input_data, diffable):
                 tags=",".join(included_datafile.tags),
             )
 
+    if report_input_data["included_metadata"]:
+        metadata_node = TreeXML.appendTreeElement(
+            root,
+            "metadata",
+        )
+
+        for distribution_name, reasons in sorted(
+            report_input_data["included_metadata"].items()
+        ):
+            TreeXML.appendTreeElement(
+                metadata_node,
+                "included_metadata",
+                name=distribution_name,
+                reason=". ".join(reasons),
+            )
+
     for standalone_entry_point in getStandaloneEntryPoints():
         if standalone_entry_point.kind == "executable":
             continue
@@ -538,10 +650,20 @@ def writeCompilationReport(report_filename, report_input_data, diffable):
             source_path=_getCompilationReportPath(standalone_entry_point.source_path),
             package=standalone_entry_point.package_name or "",
             ignored="yes" if ignored else "no",
-            reason=standalone_entry_point.reason
+            reason=standalone_entry_point.reason,
             # TODO: No reason yet.
         )
 
+    for standalone_entry_point, (reason, removed_dll_paths) in getRemovedUsedDllsInfo():
+        for removed_dll_path in removed_dll_paths:
+            TreeXML.appendTreeElement(
+                root,
+                "excluded_dll",
+                name=_getCompilationReportPath(removed_dll_path),
+                used_by=standalone_entry_point.dest_path,
+                reason=reason,
+            )
+
     if not diffable:
         data_composer_values = getDataComposerReportValues()
 
@@ -615,7 +737,9 @@ def writeCompilationReport(report_filename, report_input_data, diffable):
         python_flavor=report_input_data["python_flavor"],
         python_version=report_input_data["python_version"],
         os_name=report_input_data["os_name"],
+        os_release=report_input_data["os_release"],
         arch_name=report_input_data["arch_name"],
+        filesystem_encoding=report_input_data["filesystem_encoding"],
     )
 
     search_path = getPackageSearchPath(None)
@@ -643,10 +767,13 @@ def writeCompilationReport(report_filename, report_input_data, diffable):
         ),
     )
 
+    contents = TreeXML.toString(root)
+
+    if type(contents) is not bytes:
+        contents = contents.encode("utf8")
+
     try:
-        putTextFileContents(
-            filename=report_filename, contents=TreeXML.toString(root), encoding="utf8"
-        )
+        putBinaryFileContents(filename=report_filename, contents=contents)
     except OSError as e:
         reports_logger.warning(
             "Compilation report write to file '%s' failed due to: %s."
